pub struct AdaptiveConnectionPool { /* private fields */ }Implementations
Sourceimpl AdaptiveConnectionPool
impl AdaptiveConnectionPool
pub async fn new(db: Arc<Database>, config: AdaptivePoolConfig) -> Result<Self>
pub async fn new_sync( db: Arc<Database>, config: AdaptivePoolConfig, ) -> Result<Self>
pub async fn check_and_scale(&self)
pub async fn get(&self) -> Result<AdaptivePooledConnection>
pub fn available_connections(&self) -> usize
pub fn utilization(&self) -> f64
pub fn active_connections(&self) -> u32
pub fn max_connections(&self) -> u32
pub fn metrics(&self) -> AdaptivePoolMetrics
Sourcepub fn set_cleanup_callback(&self, callback: ConnectionCleanupCallback)
pub fn set_cleanup_callback(&self, callback: ConnectionCleanupCallback)
Register a cleanup callback to be called when connections are dropped
This allows external components (like the prepared statement cache) to clean up resources when a connection is returned to the pool.
Arguments
callback- Function to call with the connection ID when a connection is dropped
Example
use std::sync::Arc;
use do_memory_storage_turso::pool::{AdaptiveConnectionPool, ConnectionId};
use do_memory_storage_turso::PreparedStatementCache;
let cache = Arc::new(PreparedStatementCache::new(100));
let cache_clone = Arc::clone(&cache);
pool.set_cleanup_callback(Arc::new(move |conn_id: ConnectionId| {
cache_clone.clear_connection(conn_id);
}));Sourcepub fn remove_cleanup_callback(&self)
pub fn remove_cleanup_callback(&self)
Remove the cleanup callback
This disables automatic cleanup notifications.
pub async fn shutdown(&self)