AdaptivePooledConnection

Struct AdaptivePooledConnection 

pub struct AdaptivePooledConnection { /* private fields */ }

Implementations

Source

impl AdaptivePooledConnection

pub fn connection_id(&self) -> ConnectionId

Get the unique connection identifier

This ID is stable for the lifetime of the connection and can be used to associate cached data (like prepared statements) with the connection.

pub fn connection(&self) -> Option<&Connection>

Get a reference to the underlying database connection

pub fn into_inner(self) -> Option<Connection>

Take ownership of the underlying connection