pub struct AdaptivePooledConnection { /* private fields */ }Implementations
Sourceimpl AdaptivePooledConnection
impl AdaptivePooledConnection
Sourcepub fn connection_id(&self) -> ConnectionId
pub fn connection_id(&self) -> ConnectionId
Get the unique connection identifier
This ID is stable for the lifetime of the connection and can be used to associate cached data (like prepared statements) with the connection.
Sourcepub fn connection(&self) -> Option<&Connection>
pub fn connection(&self) -> Option<&Connection>
Get a reference to the underlying database connection
Sourcepub fn into_inner(self) -> Option<Connection>
pub fn into_inner(self) -> Option<Connection>
Take ownership of the underlying connection